#! /bin/gawk -f

function changeNonFileNameCharacters(sourceString) {
  return                    \
    gensub("?", ""   , "g", \
    gensub("/", "-"  , "g", sourceString))
}

function changeProblemCharacters(sourceString) {
  return                    \
    gensub("", "ae" , "g", \
    gensub("", "Ae" , "g", \
    gensub("", "oe" , "g", \
    gensub("", "Oe" , "g", \
    gensub("", "ue" , "g", \
    gensub("", "Ue" , "g", \
    gensub("", "ss" , "g", \
    gensub("", "-"  , "g", \
    gensub("", "...", "g", \
    gensub("", "\"" , "g", \
    gensub("", "\"" , "g", \
    gensub("", "a"  , "g", \
    gensub("", "e"  , "g", \
    gensub("", "o"  , "g", sourceString))))))))))))))
}

function dropContext(sourceString, prefix, suffix) {
  return gensub(suffix, "", 1, gensub(prefix, "", 1, sourceString))
}

function getAllMatches(sourceString, regex, matches,   matchesIndex) {
  while ( match(sourceString, regex) > 0 ) {
		matches[matchesIndex ++] = substr(sourceString, RSTART, RLENGTH)
		sourceString = substr(sourceString, RSTART + RLENGTH)
	}
}

function getAuthors(sourceString,   authorArray, prefix) {
  prefix = ".*<h3 class=\"author\">"
  if ( sourceString !~ prefix) return ""

  split(dropContext(sourceString, prefix, "</h3>.*"), authorArray, " / ")

  for ( key in authorArray )
    authorArray[key] = dropContext(authorArray[key], ", .*", "")

  return join(authorArray, ", ")
}

function getBookName(sourceString) {
  return changeNonFileNameCharacters(dropContext(sourceString, ".*class=\"pub-title\"><a href=\"[^>]*>", "</a>.*"))
}

function getTexts(sourceString,   key, texts, textsArray) {
  prefix = "\\.xml\\?format=EBOK\">"
  getAllMatches(withoutColon, prefix "[^<]+", textsArray)

  for ( key = 0; key < length(textsArray); ++ key ) {
    textsArray[key] = changeNonFileNameCharacters(dropContext(textsArray[key], prefix, ""))
    texts           = texts "  '" textsArray[key] "'\n"
  }

  return texts
}

function getUris(sourceString,   baseURI, books, key, separator, uriArray, uris) {
  books            = "books"
  separator        = "$002f"
  baseURI          = "http://www.degruyter.com/dg/viewbookchapter.fullcontentlink:pdfeventlink/"

  getAllMatches($0, "<li xmlns=\"http://www\\.w3\\.org/1999/xhtml\" class=\"\"><a href=\"[^?]+" , uriArray)

  for ( key = 0; key < length(uriArray); ++ key ) {
    uriArray[key] = dropContext(uriArray[key], ".*/", "\\.xml")
    bookNumber    = gensub("\\..*", "", 1, uriArray[key])
    uris          = uris "  '" baseURI separator                                                \
      books separator bookNumber separator uriArray[key] separator uriArray[key] ".pdf?t:ac="   \
      books separator bookNumber separator uriArray[key] separator uriArray[key] ".xml" "'\n"
  }

  return uris
}

function join(stringArray, glue,   result, key) {
  for ( key in stringArray )
    result = result stringArray[key] glue

  return substr(result, 1, length(result) - length(glue))
}

/class="pub-title">/ {
  $0           = changeProblemCharacters($0)
  withoutColon = gensub(":", " - ", "g", $0)
  authors      = getAuthors(withoutColon)
  bookName     = getBookName(withoutColon)
  uris         = getUris($0)
  texts        = getTexts(withoutColon)
  nextfile
}

END {
  print                                                           \
    "#! /bin/bash\n\n"                                            \
    "PATH=/bin\n\n"                                               \
    "BOOK_DIRECTORY='eBook " authors " - " bookName "'\n\n"       \
    "CHAPTER_NAMES=(\n"                                           \
    texts                                                         \
    ")\n\n"                                                       \
    "CHAPTER_URIS=(\n"                                            \
    uris                                                          \
    ")\n\n"                                                       \
    "[ -d \"$BOOK_DIRECTORY\" ] || mkdir \"$BOOK_DIRECTORY\"\n"   \
    "cd \"$BOOK_DIRECTORY\"     || exit\n\n"                      \
    "for CHAPTER_INDEX in ${!CHAPTER_NAMES[@]}\n"                 \
    "do\n"                                                        \
    "  wget                                         \\\n"         \
    "    -O \"${CHAPTER_NAMES[CHAPTER_INDEX]}.pdf\"   \\\n"       \
    "    \"${CHAPTER_URIS[CHAPTER_INDEX]}\"\n"                    \
    "done"
}
